@@ -12,7 +12,7 @@ from os.path import abspath, basename, expanduser, isdir, isfile, join, split as
 import platform
 import sys
 
-from .constants import (APP_NAME, ChannelPriority, DEFAULTS_CHANNEL_NAME,
+from .constants import (APP_NAME, ChannelPriority, DEFAULTS_CHANNEL_NAME, REPODATA_FN,
                         DEFAULT_AGGRESSIVE_UPDATE_PACKAGES, DEFAULT_CHANNELS,
                         DEFAULT_CHANNEL_ALIAS, DEFAULT_CUSTOM_CHANNELS, DepsModifier,
                         ERROR_UPLOAD_URL, PLATFORM_DIRECTORIES, PREFIX_MAGIC_FILE, PathConflict,
@@ -30,6 +30,7 @@ from ..common.configuration import (Configuration, ConfigurationLoadError, MapPa
 from ..common._os.linux import linux_get_libc_version
 from ..common.path import expand, paths_equal
 from ..common.url import has_scheme, path_to_url, split_scheme_auth_token
+from ..common.decorators import env_override
 
 from .. import CONDA_PACKAGE_ROOT
 
@@ -157,6 +158,8 @@ class Context(Configuration):
     track_features = SequenceParameter(string_types)
     use_index_cache = PrimitiveParameter(False)
 
+    separate_format_cache = PrimitiveParameter(False)
+
     _root_prefix = PrimitiveParameter("", aliases=('root_dir', 'root_prefix'))
     _envs_dirs = SequenceParameter(string_types, aliases=('envs_dirs', 'envs_path'),
                                    string_delimiter=os.pathsep,
@@ -216,6 +219,10 @@ class Context(Configuration):
     show_channel_urls = PrimitiveParameter(None, element_type=(bool, NoneType))
     use_local = PrimitiveParameter(False)
     whitelist_channels = SequenceParameter(string_types, expandvars=True)
+    restore_free_channel = PrimitiveParameter(False)
+    repodata_fns = SequenceParameter(string_types, ("current_repodata.json", REPODATA_FN))
+    _use_only_tar_bz2 = PrimitiveParameter(None, element_type=(bool, NoneType),
+                                           aliases=('use_only_tar_bz2',))
 
     always_softlink = PrimitiveParameter(False, aliases=('softlink',))
     always_copy = PrimitiveParameter(False, aliases=('copy',))
@@ -249,7 +256,6 @@ class Context(Configuration):
     # update_specs = PrimitiveParameter(False)
     # update_all = PrimitiveParameter(False)
 
-    prune = PrimitiveParameter(False)
     force_remove = PrimitiveParameter(False)
     force_reinstall = PrimitiveParameter(False)
 
@@ -266,6 +272,9 @@ class Context(Configuration):
         if search_path is None:
             search_path = SEARCH_PATH
 
+        # used to show the warning only once, and to keep the setting unset
+        self._has_warned_about_only_tar_bz2 = False
+
         if argparse_args:
             # This block of code sets CONDA_PREFIX based on '-n' and '-p' flags, so that
             # configuration can be properly loaded from those locations
@@ -543,8 +552,12 @@ class Context(Configuration):
     def custom_multichannels(self):
         from ..models.channel import Channel
 
+        default_channels = list(self._default_channels)
+        if self.restore_free_channel:
+            default_channels.insert(1, 'https://repo.anaconda.com/pkgs/free')
+
         reserved_multichannel_urls = odict((
-            (DEFAULTS_CHANNEL_NAME, self._default_channels),
+            (DEFAULTS_CHANNEL_NAME, default_channels),
             ('local', self.conda_build_local_urls),
         ))
         reserved_multichannels = odict(
@@ -607,6 +620,26 @@ class Context(Configuration):
                                                 (DEFAULTS_CHANNEL_NAME,))))
         return tuple(IndexedSet(concatv(local_add, self._channels)))
 
+    @property
+    def use_only_tar_bz2(self):
+        from ..models.version import VersionOrder
+        use_only_tar_bz2 = False
+        if self._use_only_tar_bz2 is None:
+            try:
+                import conda_build
+                use_only_tar_bz2 = VersionOrder(conda_build.__version__) < VersionOrder("3.18.3")
+                if use_only_tar_bz2 and not self._has_warned_about_only_tar_bz2:
+                    log.warn("Conda is constrained to only using the old .tar.bz2 file format "
+                             "because you have conda-build installed, and it is <3.18.3.  Update "
+                             "or remove conda-build to get smaller downloads and faster "
+                             "extractions.")
+                    self._has_warned_about_only_tar_bz2 = True
+            except ImportError:
+                pass
+            if self._argparse_args and 'use_only_tar_bz2' in self._argparse_args:
+                use_only_tar_bz2 &= self._argparse_args['use_only_tar_bz2']
+        return self._use_only_tar_bz2 or use_only_tar_bz2
+
     @property
     def binstar_upload(self):
         # backward compatibility for conda-build
@@ -691,6 +724,12 @@ class Context(Configuration):
         info = _get_cpu_info()
         return info['flags']
 
+    @memoizedproperty
+    @env_override('CONDA_OVERRIDE_CUDA', convert_empty_to_none=True)
+    def cuda_version(self):
+        from conda.common.cuda import cuda_detect
+        return cuda_detect()
+
     @property
     def category_map(self):
         return odict((
@@ -706,6 +745,9 @@ class Context(Configuration):
             'migrated_custom_channels',
             'add_anaconda_token',
             'allow_non_channel_urls',
+            'restore_free_channel',
+            'repodata_fns',
+            'use_only_tar_bz2',
         )),
         ('Basic Conda Configuration', (  # TODO: Is there a better category name here?
             'envs_dirs',
@@ -731,7 +773,6 @@ class Context(Configuration):
             'force_reinstall',
             'pinned_packages',
             'pip_interop_enabled',
-            'prune',
             'track_features',
         )),
         ('Package Linking and Install-time Configuration', (
@@ -744,6 +785,7 @@ class Context(Configuration):
             'extra_safety_checks',
             'shortcuts',
             'non_admin_enabled',
+            'separate_format_cache'
         )),
         ('Conda-build Configuration', (
             'bld_path',
@@ -1031,10 +1073,6 @@ class Context(Configuration):
                 'scheme://[user:password@]host[:port]'. The optional 'user:password' inclusion
                 enables HTTP Basic Auth with your proxy.
                 """),
-            'prune': dals("""
-                Remove packages that have previously been brought into an environment to satisfy
-                dependencies of user-requested packages, but are no longer needed.
-                """),
             'quiet': dals("""
                 Disable progress bar display and other output.
                 """),
@@ -1055,6 +1093,10 @@ class Context(Configuration):
                 reports are anonymous, with only the error stack trace and information given
                 by `conda info` being sent.
                 """),
+            'restore_free_channel': dals(""""
+                Add the "free" channel back into defaults, behind "main" in priority. The "free"
+                channel was removed from the collection of default channels in conda 4.7.0.
+            """),
             'rollback_enabled': dals("""
                 Should any error occur during an unlink/link transaction, revert any disk
                 mutations made to that point in the transaction.
@@ -1063,6 +1105,14 @@ class Context(Configuration):
                 Enforce available safety guarantees during package installation.
                 The value must be one of 'enabled', 'warn', or 'disabled'.
                 """),
+            'separate_format_cache': dals("""
+                Treat .tar.bz2 files as different from .conda packages when
+                filenames are otherwise similar. This defaults to False, so
+                that your package cache doesn't churn when rolling out the new
+                package format. If you'd rather not assume that a .tar.bz2 and
+                .conda from the same place represent the same content, set this
+                to True.
+            """),
             'extra_safety_checks': dals("""
                 Spend extra time validating package contents.  Currently, runs sha256 verification
                 on every file within each package during installation.
@@ -1086,6 +1136,13 @@ class Context(Configuration):
                 A list of features that are tracked by default. An entry here is similar to
                 adding an entry to the create_default_packages list.
                 """),
+            'repodata_fns': dals("""
+                Specify filenames for repodata fetching. The default is ('current_repodata.json',
+                'repodata.json'), which tries a subset of the full index containing only the
+                latest version for each package, then falls back to repodata.json.  You may
+                want to specify something else to use an alternate index that has been reduced
+                somehow.
+                """),
             'use_index_cache': dals("""
                 Use cache of channel index files, even if it has expired.
                 """),
@@ -1098,6 +1155,11 @@ class Context(Configuration):
                 allowed, along with the --use-local command line flag, be sure to include the
                 'local' channel in the list. If the list is empty or left undefined, no
                 channel exclusions will be enforced.
+                """),
+            'use_only_tar_bz2': dals("""
+                A boolean indicating that only .tar.bz2 conda packages should be downloaded.
+                This is forced to True if conda-build is installed and older than 3.18.3,
+                because older versions of conda break when conda feeds it the new file format.
                 """)
         })
 
