@@ -1437,6 +1437,17 @@ def add_parser_channels(p):
         action="store_true",
         help="""Do not search default or .condarc channels.  Requires --channel.""",
     )
+    channel_customization_options.add_argument(
+        "--repodata-fn",
+        action="append",
+        dest="repodata_fns",
+        help=("Specify name of repodata on remote server. Conda will try "
+              "whatever you specify, but will ultimately fall back to repodata.json if "
+              "your specs are not satisfiable with what you specify here. This is used "
+              "to employ repodata that is reduced in time scope.  You may pass this flag"
+              "more than once.  Leftmost entries are tried first, and the fallback to"
+              "repodata.json is added for you automatically.")
+    )
     return channel_customization_options
 
 
@@ -1538,8 +1549,7 @@ def add_parser_prune(p):
         "--prune",
         action="store_true",
         default=NULL,
-        help="Remove packages that have previously been brought into the environment to satisfy "
-             "dependencies of user-requested packages, but are no longer needed.",
+        help=SUPPRESS,
     )
 
 
