@@ -11,10 +11,11 @@ from . import common
 from .common import check_non_admin
 from .. import CondaError
 from .._vendor.auxlib.ish import dals
-from ..base.constants import ROOT_ENV_NAME, UpdateModifier
+from ..base.constants import ROOT_ENV_NAME, UpdateModifier, REPODATA_FN
 from ..base.context import context, locate_prefix_by_name
 from ..common.compat import on_win, text_type
-from ..common.path import paths_equal
+from ..common.constants import NULL
+from ..common.path import paths_equal, is_package_file
 from ..core.index import calculate_channel_urls, get_index
 from ..core.prefix_data import PrefixData
 from ..core.solve import DepsModifier, Solver
@@ -22,7 +23,8 @@ from ..exceptions import (CondaExitZero, CondaImportError, CondaOSError, CondaSy
                           CondaValueError, DirectoryNotACondaEnvironmentError,
                           DirectoryNotFoundError, DryRunExit, EnvironmentLocationNotFound,
                           NoBaseEnvironmentError, PackageNotInstalledError, PackagesNotFoundError,
-                          TooManyArgumentsError, UnsatisfiableError)
+                          TooManyArgumentsError, UnsatisfiableError,
+                          SpecsConfigurationConflictError)
 from ..gateways.disk.create import mkdir_p
 from ..gateways.disk.delete import delete_trash, path_is_clean
 from ..misc import clone_env, explicit, touch_nonadmin
@@ -124,17 +126,22 @@ def get_revision(arg, json=False):
     except ValueError:
         raise CondaValueError("expected revision number, not: '%s'" % arg, json)
 
-
 def install(args, parser, command='install'):
     """
     conda install, conda update, and conda create
     """
     context.validate_configuration()
     check_non_admin()
+    # this is sort of a hack.  current_repodata.json may not have any .tar.bz2 files,
+    #    because it deduplicates records that exist as both formats.  Forcing this to
+    #    repodata.json ensures that .tar.bz2 files are available
+    if context.use_only_tar_bz2:
+        args.repodata_fns = ('repodata.json', )
 
     newenv = bool(command == 'create')
     isupdate = bool(command == 'update')
     isinstall = bool(command == 'install')
+    isremove = bool(command == 'remove')
     if newenv:
         common.ensure_name_or_prefix(args, command)
     prefix = context.target_prefix
@@ -189,7 +196,7 @@ def install(args, parser, command='install'):
         'use_local': args.use_local
     }
 
-    num_cp = sum(s.endswith('.tar.bz2') for s in args_packages)
+    num_cp = sum(is_package_file(s) for s in args_packages)
     if num_cp:
         if num_cp == len(args_packages):
             explicit(args_packages, prefix, verbose=not context.quiet)
@@ -239,39 +246,61 @@ def install(args, parser, command='install'):
         print_activate(args.name if args.name else prefix)
         return
 
-    try:
-        if isinstall and args.revision:
-            index = get_index(channel_urls=index_args['channel_urls'],
-                              prepend=index_args['prepend'], platform=None,
-                              use_local=index_args['use_local'], use_cache=index_args['use_cache'],
-                              unknown=index_args['unknown'], prefix=prefix)
-            unlink_link_transaction = revert_actions(prefix, get_revision(args.revision), index)
-        else:
-            if isupdate:
-                deps_modifier = context.deps_modifier or DepsModifier.UPDATE_SPECS
+    repodata_fns = args.repodata_fns
+    if not repodata_fns:
+        repodata_fns = ["current_repodata.json", "repodata.json"]
+    elif REPODATA_FN not in repodata_fns:
+        repodata_fns.append(REPODATA_FN)
+
+    for repodata_fn in repodata_fns:
+        try:
+            update_modifier = context.update_modifier
+            if isinstall and args.revision:
+                index = get_index(channel_urls=index_args['channel_urls'],
+                                  prepend=index_args['prepend'], platform=None,
+                                  use_local=index_args['use_local'],
+                                  use_cache=index_args['use_cache'],
+                                  unknown=index_args['unknown'], prefix=prefix,
+                                  repodata_fn=repodata_fn)
+                unlink_link_transaction = revert_actions(prefix, get_revision(args.revision),
+                                                         index)
             else:
-                deps_modifier = context.deps_modifier
-            solver = Solver(prefix, context.channels, context.subdirs, specs_to_add=specs)
-            unlink_link_transaction = solver.solve_for_transaction(
-                deps_modifier=deps_modifier,
-                force_reinstall=context.force_reinstall or context.force,
-            )
+                solver = Solver(prefix, context.channels, context.subdirs, specs_to_add=specs,
+                                repodata_fn=repodata_fn)
+                if (isinstall or isremove) and args.update_modifier == NULL:
+                    # try to do a quick solve with then existing env frozen by default
+                    update_modifier = UpdateModifier.FREEZE_INSTALLED
+                if isupdate:
+                    deps_modifier = context.deps_modifier or DepsModifier.UPDATE_SPECS
+                else:
+                    deps_modifier = context.deps_modifier
+                unlink_link_transaction = solver.solve_for_transaction(
+                    deps_modifier=deps_modifier,
+                    update_modifier=update_modifier,
+                    force_reinstall=context.force_reinstall or context.force,
+                )
+            # we only need one of these to work.  If we haven't raised an exception,
+            #   we're good.
+            break
 
-    except ResolvePackageNotFound as e:
-        channels_urls = tuple(calculate_channel_urls(
-            channel_urls=index_args['channel_urls'],
-            prepend=index_args['prepend'],
-            platform=None,
-            use_local=index_args['use_local'],
-        ))
-        raise PackagesNotFoundError(e._formatted_chains, channels_urls)
-
-    except (UnsatisfiableError, SystemExit) as e:
-        # Unsatisfiable package specifications/no such revision/import error
-        if e.args and 'could not import' in e.args[0]:
-            raise CondaImportError(text_type(e))
-        raise
+        except (ResolvePackageNotFound, PackagesNotFoundError) as e:
+            channels_urls = tuple(calculate_channel_urls(
+                channel_urls=index_args['channel_urls'],
+                prepend=index_args['prepend'],
+                platform=None,
+                use_local=index_args['use_local'],
+            ))
+            # end of the line.  Raise the exception
+            if repodata_fn == repodata_fns[-1]:
+                raise PackagesNotFoundError(e._formatted_chains, channels_urls)
 
+        except (UnsatisfiableError, SystemExit, SpecsConfigurationConflictError) as e:
+            # end of the line.  Raise the exception
+            if repodata_fn == repodata_fns[-1]:
+                # Unsatisfiable package specifications/no such revision/import error
+                if e.args and 'could not import' in e.args[0]:
+                    raise CondaImportError(text_type(e))
+                raise
     handle_txn(unlink_link_transaction, prefix, args, newenv)
 
 
