@@ -17,7 +17,7 @@ def execute(args, parser):
 
     call = args.executable_call
     cwd = args.cwd
-    prefix = args.prefix or os.getenv("CONDA_PREFIX") or context.root_prefix
+    prefix = context.target_prefix or os.getenv("CONDA_PREFIX") or context.root_prefix
 
     script_caller, command_args = wrap_subprocess_call(on_win, context.root_prefix, prefix,
                                                        args.dev, args.debug_wrapper_scripts, call)
