@@ -4,10 +4,10 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 from abc import ABCMeta, abstractmethod, abstractproperty
-from errno import EXDEV
 from logging import getLogger
-from os.path import basename, dirname, getsize, join, isdir
+from os.path import basename, dirname, getsize, isdir, join
 import re
+import sys
 from uuid import uuid4
 
 from .envs_manager import get_user_environments_txt_file, register_env, unregister_env
@@ -17,17 +17,16 @@ from .. import CondaError
 from .._vendor.auxlib.compat import with_metaclass
 from .._vendor.auxlib.ish import dals
 from .._vendor.toolz import concat
-from ..base.constants import CONDA_TARBALL_EXTENSION
+from ..base.constants import CONDA_TEMP_EXTENSION
 from ..base.context import context
-from ..common.compat import iteritems, on_win, text_type
-from ..common.constants import CONDA_TEMP_EXTENSION
+from ..common.compat import iteritems, on_win, text_type, JSONDecodeError
 from ..common.path import (get_bin_directory_short_path, get_leaf_directories,
                            get_python_noarch_target_path, get_python_short_path,
                            parse_entry_point_def,
                            pyc_path, url_to_path, win_path_ok)
 from ..common.url import has_platform, path_to_url
-from ..exceptions import (CondaUpgradeError, CondaVerificationError, PaddingError, SafetyError,
-                          NotWritableError)
+from ..exceptions import (CondaUpgradeError, CondaVerificationError, NotWritableError,
+                          PaddingError, SafetyError)
 from ..gateways.connection.download import download
 from ..gateways.disk.create import (compile_multiple_pyc, copy,
                                     create_hard_link_or_copy, create_link,
@@ -45,6 +44,10 @@ from ..models.match_spec import MatchSpec
 from ..models.records import (Link, PackageCacheRecord, PackageRecord, PathDataV1, PathsData,
                               PrefixRecord)
 
+try:
+    FileNotFoundError
+except NameError:
+    FileNotFoundError = IOError
 
 log = getLogger(__name__)
 
@@ -206,6 +209,11 @@ class LinkPathAction(CreateInPrefixPathAction):
         def make_file_link_action(source_path_data):
             # TODO: this inner function is still kind of a mess
             noarch = package_info.repodata_record.noarch
+            if noarch is None and package_info.package_metadata is not None:
+                # Look in package metadata in case it was omitted from repodata (see issue #8311)
+                noarch = package_info.package_metadata.noarch
+                if noarch is not None:
+                    noarch = noarch.type
             if noarch == NoarchType.python:
                 sp_dir = transaction_context['target_site_packages_short_path']
                 if sp_dir is None:
@@ -502,7 +510,8 @@ class CompileMultiPycAction(MultiPathAction):
             noarch_py_file_re = re.compile(r'^site-packages[/\\][^\t\n\r\f\v]+\.py$')
             py_ver = transaction_context['target_python_version']
             py_files = tuple((axn.target_short_path for axn in file_link_actions
-                              if noarch_py_file_re.match(axn.source_short_path)))
+                              if getattr(axn, 'source_short_path') and
+                              noarch_py_file_re.match(axn.source_short_path)))
             pyc_files = tuple((pyc_path(pf, py_ver) for pf in py_files))
             return (cls(transaction_context, package_info, target_prefix, py_files, pyc_files), )
         else:
@@ -621,7 +630,9 @@ class CreatePythonEntryPointAction(CreateInPrefixPathAction):
         else:
             target_python_version = self.transaction_context['target_python_version']
             python_short_path = get_python_short_path(target_python_version)
-            python_full_path = join(self.target_prefix, win_path_ok(python_short_path))
+            python_full_path = join(
+                context.target_prefix_override or self.target_prefix,
+                win_path_ok(python_short_path))
 
         create_python_entry_point(self.target_full_path, python_full_path,
                                   self.module, self.func)
@@ -818,7 +829,7 @@ class CreatePrefixRecordAction(CreateInPrefixPathAction):
                                                        target_short_path)
         self.requested_link_type = requested_link_type
         self.requested_spec = requested_spec
-        self.all_link_path_actions = all_link_path_actions
+        self.all_link_path_actions = list(all_link_path_actions)
         self._execute_successful = False
 
     def execute(self):
@@ -827,25 +838,28 @@ class CreatePrefixRecordAction(CreateInPrefixPathAction):
             type=self.requested_link_type,
         )
         extracted_package_dir = self.package_info.extracted_package_dir
-        package_tarball_full_path = extracted_package_dir + CONDA_TARBALL_EXTENSION
-        # TODO: don't make above assumption; put package_tarball_full_path in package_info
+        package_tarball_full_path = self.package_info.package_tarball_full_path
 
         def files_from_action(link_path_action):
             if isinstance(link_path_action, CompileMultiPycAction):
                 return link_path_action.target_short_paths
             else:
-                return (link_path_action.target_short_path, )
+                return ((link_path_action.target_short_path, )
+                        if isinstance(link_path_action, CreateInPrefixPathAction) and
+                        (not hasattr(link_path_action, 'link_type') or
+                         link_path_action.link_type != LinkType.directory) else ())
 
         def paths_from_action(link_path_action):
             if isinstance(link_path_action, CompileMultiPycAction):
                 return link_path_action.prefix_paths_data
             else:
-                if link_path_action.prefix_path_data is None:
+                if (not hasattr(link_path_action, 'prefix_path_data') or
+                        link_path_action.prefix_path_data is None):
                     return ()
                 else:
                     return (link_path_action.prefix_path_data, )
 
-        files = concat((files_from_action(x) for x in self.all_link_path_actions if x))
+        files = list(concat(files_from_action(x) for x in self.all_link_path_actions if x))
         paths_data = PathsData(
             paths_version=1,
             paths=concat((paths_from_action(x) for x in self.all_link_path_actions if x)),
@@ -1065,12 +1079,13 @@ class UnregisterEnvironmentLocationAction(PathAction):
 class CacheUrlAction(PathAction):
 
     def __init__(self, url, target_pkgs_dir, target_package_basename,
-                 md5sum=None, expected_size_in_bytes=None):
+                 sha256=None, size=None, md5=None):
         self.url = url
         self.target_pkgs_dir = target_pkgs_dir
         self.target_package_basename = target_package_basename
-        self.md5sum = md5sum
-        self.expected_size_in_bytes = expected_size_in_bytes
+        self.sha256 = sha256
+        self.size = size
+        self.md5 = md5
         self.hold_path = self.target_full_path + CONDA_TEMP_EXTENSION
 
     def verify(self):
@@ -1097,54 +1112,72 @@ class CacheUrlAction(PathAction):
 
         if self.url.startswith('file:/'):
             source_path = url_to_path(self.url)
-            if dirname(source_path) in context.pkgs_dirs:
-                # if url points to another package cache, link to the writable cache
-                create_hard_link_or_copy(source_path, self.target_full_path)
-                source_package_cache = PackageCacheData(dirname(source_path))
-
-                # the package is already in a cache, so it came from a remote url somewhere;
-                #   make sure that remote url is the most recent url in the
-                #   writable cache urls.txt
-                origin_url = source_package_cache._urls_data.get_url(self.target_package_basename)
-                if origin_url and has_platform(origin_url, context.known_subdirs):
-                    target_package_cache._urls_data.add_url(origin_url)
-            else:
-                # so our tarball source isn't a package cache, but that doesn't mean it's not
-                #   in another package cache somewhere
-                # let's try to find the actual, remote source url by matching md5sums, and then
-                #   record that url as the remote source url in urls.txt
-                # we do the search part of this operation before the create_link so that we
-                #   don't md5sum-match the file created by 'create_link'
-                # there is no point in looking for the tarball in the cache that we are writing
-                #   this file into because we have already removed the previous file if there was
-                #   any. This also makes sure that we ignore the md5sum of a possible extracted
-                #   directory that might exist in this cache because we are going to overwrite it
-                #   anyway when we extract the tarball.
-                source_md5sum = compute_md5sum(source_path)
-                exclude_caches = self.target_pkgs_dir,
-                pc_entry = PackageCacheData.tarball_file_in_cache(source_path, source_md5sum,
-                                                                  exclude_caches=exclude_caches)
-
-                if pc_entry:
-                    origin_url = target_package_cache._urls_data.get_url(
-                        pc_entry.extracted_package_dir
-                    )
-                else:
-                    origin_url = None
-
-                # copy the tarball to the writable cache
-                create_link(source_path, self.target_full_path, link_type=LinkType.copy,
-                            force=context.force)
-
-                if origin_url and has_platform(origin_url, context.known_subdirs):
-                    target_package_cache._urls_data.add_url(origin_url)
-                else:
-                    target_package_cache._urls_data.add_url(self.url)
-
+            self._execute_local(source_path, target_package_cache, progress_update_callback)
         else:
-            download(self.url, self.target_full_path, self.md5sum,
-                     progress_update_callback=progress_update_callback)
-            target_package_cache._urls_data.add_url(self.url)
+            self._execute_channel(target_package_cache, progress_update_callback)
+
+    def _execute_local(self, source_path, target_package_cache, progress_update_callback=None):
+        from .package_cache_data import PackageCacheData
+        if dirname(source_path) in context.pkgs_dirs:
+            # if url points to another package cache, link to the writable cache
+            create_hard_link_or_copy(source_path, self.target_full_path)
+            source_package_cache = PackageCacheData(dirname(source_path))
+
+            # the package is already in a cache, so it came from a remote url somewhere;
+            #   make sure that remote url is the most recent url in the
+            #   writable cache urls.txt
+            origin_url = source_package_cache._urls_data.get_url(self.target_package_basename)
+            if origin_url and has_platform(origin_url, context.known_subdirs):
+                target_package_cache._urls_data.add_url(origin_url)
+        else:
+            # so our tarball source isn't a package cache, but that doesn't mean it's not
+            #   in another package cache somewhere
+            # let's try to find the actual, remote source url by matching md5sums, and then
+            #   record that url as the remote source url in urls.txt
+            # we do the search part of this operation before the create_link so that we
+            #   don't md5sum-match the file created by 'create_link'
+            # there is no point in looking for the tarball in the cache that we are writing
+            #   this file into because we have already removed the previous file if there was
+            #   any. This also makes sure that we ignore the md5sum of a possible extracted
+            #   directory that might exist in this cache because we are going to overwrite it
+            #   anyway when we extract the tarball.
+            source_md5sum = compute_md5sum(source_path)
+            exclude_caches = self.target_pkgs_dir,
+            pc_entry = PackageCacheData.tarball_file_in_cache(
+                source_path, source_md5sum, exclude_caches=exclude_caches
+            )
+
+            if pc_entry:
+                origin_url = target_package_cache._urls_data.get_url(
+                    pc_entry.extracted_package_dir
+                )
+            else:
+                origin_url = None
+
+            # copy the tarball to the writable cache
+            create_link(source_path, self.target_full_path, link_type=LinkType.copy,
+                        force=context.force)
+
+            if origin_url and has_platform(origin_url, context.known_subdirs):
+                target_package_cache._urls_data.add_url(origin_url)
+            else:
+                target_package_cache._urls_data.add_url(self.url)
+
+    def _execute_channel(self, target_package_cache, progress_update_callback=None):
+        kwargs = {}
+        if self.size is not None:
+            kwargs["size"] = self.size
+        if self.sha256:
+            kwargs["sha256"] = self.sha256
+        elif self.md5:
+            kwargs["md5"] = self.md5
+        download(
+            self.url,
+            self.target_full_path,
+            progress_update_callback=progress_update_callback,
+            **kwargs
+        )
+        target_package_cache._urls_data.add_url(self.url)
 
     def reverse(self):
         if lexists(self.hold_path):
@@ -1165,13 +1198,15 @@ class CacheUrlAction(PathAction):
 class ExtractPackageAction(PathAction):
 
     def __init__(self, source_full_path, target_pkgs_dir, target_extracted_dirname,
-                 record_or_spec, md5sum):
+                 record_or_spec, sha256, size, md5):
         self.source_full_path = source_full_path
         self.target_pkgs_dir = target_pkgs_dir
         self.target_extracted_dirname = target_extracted_dirname
         self.hold_path = self.target_full_path + CONDA_TEMP_EXTENSION
         self.record_or_spec = record_or_spec
-        self.md5sum = md5sum
+        self.sha256 = sha256
+        self.size = size
+        self.md5 = md5
 
     def verify(self):
         self._verified = True
@@ -1182,35 +1217,36 @@ class ExtractPackageAction(PathAction):
         from .package_cache_data import PackageCacheData
         log.trace("extracting %s => %s", self.source_full_path, self.target_full_path)
 
-        if lexists(self.hold_path):
-            rm_rf(self.hold_path)
         if lexists(self.target_full_path):
-            try:
-                backoff_rename(self.target_full_path, self.hold_path)
-            except (IOError, OSError) as e:
-                if e.errno == EXDEV:
-                    # OSError(18, 'Invalid cross-device link')
-                    # https://github.com/docker/docker/issues/25409
-                    # ignore, but we won't be able to roll back
-                    log.debug("Invalid cross-device link on rename %s => %s",
-                              self.target_full_path, self.hold_path)
-                    rm_rf(self.target_full_path)
-                else:
-                    raise
+            rm_rf(self.target_full_path)
 
         extract_tarball(self.source_full_path, self.target_full_path,
                         progress_update_callback=progress_update_callback)
 
-        raw_index_json = read_index_json(self.target_full_path)
+        try:
+            raw_index_json = read_index_json(self.target_full_path)
+        except (IOError, OSError, JSONDecodeError, FileNotFoundError):
+            # At this point, we can assume the package tarball is bad.
+            # Remove everything and move on.
+            print("ERROR: Encountered corrupt package tarball at %s. Conda has "
+                  "left it in place.  Please report this to the maintainers "
+                  "of your package.  For the defaults channel, please report "
+                  "to https://github.com/continuumio/anaconda-issues" % self.source_full_path)
+            sys.exit(1)
 
         if isinstance(self.record_or_spec, MatchSpec):
             url = self.record_or_spec.get_raw_value('url')
             assert url
             channel = Channel(url) if has_platform(url, context.known_subdirs) else Channel(None)
             fn = basename(url)
-            md5 = self.md5sum or compute_md5sum(self.source_full_path)
-            repodata_record = PackageRecord.from_objects(raw_index_json, url=url,
-                                                         channel=channel, fn=fn, md5=md5)
+            sha256 = self.sha256 or compute_sha256sum(self.source_full_path)
+            size = getsize(self.source_full_path)
+            if self.size is not None:
+                assert size == self.size, (size, self.size)
+            md5 = self.md5 or compute_md5sum(self.source_full_path)
+            repodata_record = PackageRecord.from_objects(
+                raw_index_json, url=url, channel=channel, fn=fn, sha256=sha256, size=size, md5=md5,
+            )
         else:
             repodata_record = PackageRecord.from_objects(self.record_or_spec, raw_index_json)
 
@@ -1225,10 +1261,6 @@ class ExtractPackageAction(PathAction):
         )
         target_package_cache.insert(package_cache_record)
 
-        # dist = Dist(recorded_url) if recorded_url else Dist(path_to_url(self.source_full_path))
-        # package_cache_entry = PackageCacheRecord.make_legacy(self.target_pkgs_dir, dist)
-        # target_package_cache[package_cache_entry.dist] = package_cache_entry
-
     def reverse(self):
         rm_rf(self.target_full_path)
         if lexists(self.hold_path):
