@@ -3,13 +3,14 @@
 # SPDX-License-Identifier: BSD-3-Clause
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+import copy
 from genericpath import exists
 from logging import DEBUG, getLogger
 from os.path import join
 import sys
 from textwrap import dedent
 
-from .index import get_reduced_index
+from .index import get_reduced_index, _supplement_index_with_system
 from .link import PrefixSetup, UnlinkLinkTransaction
 from .prefix_data import PrefixData
 from .subdir_data import SubdirData
@@ -18,14 +19,14 @@ from .._vendor.auxlib.decorators import memoizedproperty
 from .._vendor.auxlib.ish import dals
 from .._vendor.boltons.setutils import IndexedSet
 from .._vendor.toolz import concat, concatv, groupby
-from ..base.constants import DepsModifier, UNKNOWN_CHANNEL, UpdateModifier
+from ..base.constants import DepsModifier, UNKNOWN_CHANNEL, UpdateModifier, REPODATA_FN
 from ..base.context import context
 from ..common.compat import iteritems, itervalues, odict, text_type
 from ..common.constants import NULL
 from ..common.io import Spinner, dashlist, time_recorder
 from ..common.path import get_major_minor_version, paths_equal
-from ..exceptions import PackagesNotFoundError, SpecsConfigurationConflictError
-from ..gateways.logging import TRACE
+from ..exceptions import (PackagesNotFoundError, SpecsConfigurationConflictError,
+                          ResolvePackageNotFound)
 from ..history import History
 from ..models.channel import Channel
 from ..models.enums import NoarchType
@@ -48,7 +49,8 @@ class Solver(object):
 
     """
 
-    def __init__(self, prefix, channels, subdirs=(), specs_to_add=(), specs_to_remove=()):
+    def __init__(self, prefix, channels, subdirs=(), specs_to_add=(), specs_to_remove=(),
+                 repodata_fn=REPODATA_FN):
         """
         Args:
             prefix (str):
@@ -68,12 +70,15 @@ class Solver(object):
         self.channels = IndexedSet(Channel(c) for c in channels or context.channels)
         self.subdirs = tuple(s for s in subdirs or context.subdirs)
         self.specs_to_add = frozenset(MatchSpec.merge(s for s in specs_to_add))
+        self.specs_to_add_names = frozenset(_.name for _ in self.specs_to_add)
         self.specs_to_remove = frozenset(MatchSpec.merge(s for s in specs_to_remove))
 
         assert all(s in context.known_subdirs for s in self.subdirs)
+        self._repodata_fn = repodata_fn
         self._index = None
         self._r = None
         self._prepared = False
+        self._pool_cache = {}
 
     def solve_for_transaction(self, update_modifier=NULL, deps_modifier=NULL, prune=NULL,
                               ignore_pinned=NULL, force_remove=NULL, force_reinstall=NULL):
@@ -169,11 +174,11 @@ class Solver(object):
                 default solver behavior is to be as conservative as possible with dependency
                 updates (in the case the dependency already exists in the environment), while
                 still ensuring all dependencies are satisfied.  Options include
-                    * NO_DEPS
-                    * ONLY_DEPS
-                    * UPDATE_DEPS
-                    * UPDATE_DEPS_ONLY_DEPS
-                    * FREEZE_INSTALLED
+                * NO_DEPS
+                * ONLY_DEPS
+                * UPDATE_DEPS
+                * UPDATE_DEPS_ONLY_DEPS
+                * FREEZE_INSTALLED
             prune (bool):
                 If ``True``, the solution will not contain packages that were
                 previously brought into the environment as dependencies but are no longer
@@ -198,7 +203,6 @@ class Solver(object):
             deps_modifier = context.deps_modifier
         else:
             deps_modifier = DepsModifier(text_type(deps_modifier).lower())
-        prune = context.prune if prune is NULL else prune
         ignore_pinned = context.ignore_pinned if ignore_pinned is NULL else ignore_pinned
         force_remove = context.force_remove if force_remove is NULL else force_remove
 
@@ -230,16 +234,25 @@ class Solver(object):
                 # Return early, with a solution that should just be PrefixData().iter_records()
                 return IndexedSet(PrefixGraph(ssc.solution_precs).graph)
 
-        with Spinner("Collecting package metadata", not context.verbosity and not context.quiet,
+        with Spinner("Collecting package metadata (%s)" % self._repodata_fn,
+                     not context.verbosity and not context.quiet,
                      context.json):
             ssc = self._collect_all_metadata(ssc)
 
         with Spinner("Solving environment", not context.verbosity and not context.quiet,
                      context.json):
             ssc = self._remove_specs(ssc)
-            ssc = self._find_inconsistent_packages(ssc)
             ssc = self._add_specs(ssc)
+            solution_precs = copy.copy(ssc.solution_precs)
+            ssc = self._find_inconsistent_packages(ssc)
+            # this will prune precs that are deps of precs that get removed due to conflicts
             ssc = self._run_sat(ssc)
+            # if there were any conflicts, we need to add their orphaned deps back in
+            if ssc.add_back_map:
+                orphan_precs = (set(solution_precs)
+                                - set(ssc.solution_precs)
+                                - set(ssc.add_back_map))
+                ssc.solution_precs.extend([_ for _ in orphan_precs if _.name not in ssc.specs_map])
             ssc = self._post_sat_handling(ssc)
             ssc = self._check_solution(ssc)
 
@@ -255,28 +268,29 @@ class Solver(object):
 
     @time_recorder(module_name=__name__)
     def _collect_all_metadata(self, ssc):
-        if ssc.prune:  # or update_modifier == UpdateModifier.UPDATE_ALL  # pending conda/constructor#138  # NOQA
-            # Users are struggling with the prune functionality in --update-all, due to
-            # https://github.com/conda/constructor/issues/138.  Until that issue is resolved,
-            # and for the foreseeable future, it's best to be more conservative with --update-all.
-
-            # Start with empty specs map for UPDATE_ALL because we're optimizing the update
-            # only for specs the user has requested; it's ok to remove dependencies.
-
-            # However, because of https://github.com/conda/constructor/issues/138, we need
-            # to hard-code keeping conda, conda-build, and anaconda, if they're already in
-            # the environment.
-            for pkg_name in ('anaconda', 'conda', 'conda-build'):
-                if ssc.prefix_data.get(pkg_name, None):
-                    ssc.specs_from_history_map[pkg_name] = MatchSpec(pkg_name)
-        else:
-            ssc.specs_map.update(
-                (prec.name, MatchSpec(prec.name)) for prec in ssc.prefix_data.iter_records()
-            )
-
         # add in historically-requested specs
         ssc.specs_map.update(ssc.specs_from_history_map)
 
+        for pkg_name in ('anaconda', 'conda', 'conda-build'):
+            if pkg_name not in ssc.specs_map and ssc.prefix_data.get(pkg_name, None):
+                ssc.specs_map[pkg_name] = MatchSpec(pkg_name)
+
+        for prec in ssc.prefix_data.iter_records():
+            # first check: add everything if we have no history to work with.
+            #    This happens with "update --all", for example.
+            #
+            # second check: add in aggressively updated packages
+            #
+            # third check: add in foreign stuff (e.g. from pip) into the specs
+            #    map. We add it so that it can be left alone more. This is a
+            #    declaration that it is manually installed, much like the
+            #    history map. It may still be replaced if it is in conflict,
+            #    but it is not just an indirect dep that can be pruned.
+            if (not ssc.specs_from_history_map
+                    or MatchSpec(prec.name) in context.aggressive_update_packages
+                    or prec.subdir == 'pypi'):
+                ssc.specs_map.update({prec.name: MatchSpec(prec.name)})
+
         prepared_specs = set(concatv(
             self.specs_to_remove,
             self.specs_to_add,
@@ -362,15 +376,89 @@ class Solver(object):
                 # pop and save matching spec in specs_map
                 spec = ssc.specs_map.pop(prec.name, None)
                 ssc.add_back_map[prec.name] = (prec, spec)
+                # let the package float.  This is essential to keep the package's dependencies
+                #    in the solution
+                ssc.specs_map[prec.name] = MatchSpec(prec.name, target=prec.dist_str())
                 # inconsistent environments should maintain the python version
                 # unless explicitly requested by the user. This along with the logic in
                 # _add_specs maintains the major.minor version
-                if prec.name == 'python':
+                if prec.name == 'python' and spec:
                     ssc.specs_map['python'] = spec
             ssc.solution_precs = tuple(prec for prec in ssc.solution_precs
                                        if prec not in inconsistent_precs)
         return ssc
 
+    def _get_package_pool(self, ssc, specs):
+        specs = frozenset(specs)
+        if specs in self._pool_cache:
+            pool = self._pool_cache[specs]
+        else:
+            pool = ssc.r.get_reduced_index(specs)
+            grouped_pool = groupby(lambda x: x.name, pool)
+            pool = {k: set(v) for k, v in iteritems(grouped_pool)}
+            self._pool_cache[specs] = pool
+        return pool
+
+    def _package_has_updates(self, ssc, spec, installed_pool):
+        installed_prec = installed_pool.get(spec.name)
+        has_update = installed_prec and any(_.version > installed_prec[0].version
+                                            for _ in ssc.r.groups.get(spec.name, []))
+        return (MatchSpec(spec.name, version=">" + str(installed_prec[0].version))
+                if has_update else None)
+
+    def _should_freeze(self, ssc, target_prec, conflict_specs, explicit_pool, installed_pool):
+        # never, ever freeze anything if we have no history.
+        if not ssc.specs_from_history_map:
+            return False
+
+        pkg_name = target_prec.name
+        # if we are FREEZE_INSTALLED (first pass for install)
+        freeze = ssc.update_modifier == UpdateModifier.FREEZE_INSTALLED
+
+        if not freeze:
+            # we are UPDATE_SPECS (solve_for_diff or update specs),
+            #       AND the spec is not an explicit spec
+            update_added = (ssc.update_modifier == UpdateModifier.UPDATE_SPECS and
+                            pkg_name not in self.specs_to_add_names)
+            # more expensive, but better check.  If anything in the package pool for this spec
+            #    overlaps with the explicit package pool (by name), but they don't share any
+            #    actual records, then this target_prec conflicts and should not be pinned
+            if update_added:
+                spec_package_pool = self._get_package_pool(ssc, (target_prec.to_match_spec(), ))
+                for spec in self.specs_to_add_names:
+                    new_explicit_pool = explicit_pool
+                    ms = MatchSpec(spec)
+                    updated_spec = self._package_has_updates(ssc, ms, installed_pool)
+                    if updated_spec:
+                        try:
+                            new_explicit_pool = self._get_package_pool(ssc, (updated_spec, ))
+                        except ResolvePackageNotFound:
+                            update_added = False
+                            break
+
+                    if ms.name in spec_package_pool:
+                        update_added &= bool(spec_package_pool[ms.name] &
+                                             new_explicit_pool[ms.name])
+                    if not update_added:
+                        break
+
+        # if all package specs have overlapping package choices (satisfiable in at least one way)
+        no_conflict = ((freeze or update_added) and
+                       pkg_name not in conflict_specs and
+                       (pkg_name not in explicit_pool or
+                        target_prec in explicit_pool[pkg_name]) and
+                       self._compare_pools(ssc, explicit_pool, target_prec.to_match_spec()))
+        return no_conflict
+
+    def _compare_pools(self, ssc, explicit_pool, ms):
+        other_pool = self._get_package_pool(ssc, (ms, ))
+        match = True
+        for k in set(other_pool.keys()) & set(explicit_pool.keys()):
+            if not bool(other_pool[k] & explicit_pool[k]):
+                match = False
+                break
+        return match
+
     def _add_specs(self, ssc):
         # For the remaining specs in specs_map, add target to each spec. `target` is a reference
         # to the package currently existing in the environment. Setting target instructs the
@@ -381,6 +469,17 @@ class Solver(object):
         # TLDR: when working with MatchSpec objects,
         #  - to minimize the version change, set MatchSpec(name=name, target=prec.dist_str())
         #  - to freeze the package, set all the components of MatchSpec individually
+
+        installed_pool = groupby(lambda x: x.name, ssc.prefix_data.iter_records())
+
+        # the only things we should consider freezing are things that don't conflict with the new
+        #    specs being added.
+        explicit_pool = self._get_package_pool(ssc, self.specs_to_add)
+
+        conflict_specs = ssc.r.get_conflicting_specs(list(concatv(
+            self.specs_to_add, (_.to_match_spec() for _ in ssc.prefix_data.iter_records())))) or []
+        conflict_specs = set(_.name for _ in conflict_specs)
+
         for pkg_name, spec in iteritems(ssc.specs_map):
             matches_for_spec = tuple(prec for prec in ssc.solution_precs if spec.match(prec))
             if matches_for_spec:
@@ -396,13 +495,46 @@ class Solver(object):
                     """) % (pkg_name, spec,
                             dashlist((text_type(s) for s in matches_for_spec), indent=4)))
                 target_prec = matches_for_spec[0]
-                if ssc.update_modifier == UpdateModifier.FREEZE_INSTALLED:
-                    new_spec = MatchSpec(target_prec)
+                if target_prec.is_unmanageable:
+                    ssc.specs_map[pkg_name] = target_prec.to_match_spec()
+                elif MatchSpec(pkg_name) in context.aggressive_update_packages:
+                    ssc.specs_map[pkg_name] = pkg_name
+                elif self._should_freeze(ssc, target_prec, conflict_specs, explicit_pool,
+                                         installed_pool):
+                    ssc.specs_map[pkg_name] = target_prec.to_match_spec()
+                elif pkg_name in ssc.specs_from_history_map:
+                    ssc.specs_map[pkg_name] = ssc.specs_from_history_map[pkg_name]
                 else:
-                    new_spec = MatchSpec(spec, target=target_prec.dist_str())
-                ssc.specs_map[pkg_name] = new_spec
-        if log.isEnabledFor(TRACE):
-            log.trace("specs_map with targets: %s", ssc.specs_map)
+                    ssc.specs_map[pkg_name] = MatchSpec(pkg_name, target=target_prec.dist_str())
+
+        pin_overrides = set()
+        for s in ssc.pinned_specs:
+            if s.name in explicit_pool:
+                if s.name not in self.specs_to_add_names and not ssc.ignore_pinned:
+                    ssc.specs_map[s.name] = MatchSpec(s, optional=False)
+                elif explicit_pool[s.name] & self._get_package_pool(ssc, [s])[s.name]:
+                    ssc.specs_map[s.name] = MatchSpec(s, optional=False)
+                    pin_overrides.add(s.name)
+                else:
+                    log.warn("pinned spec %s conflicts with explicit specs.  "
+                             "Overriding pinned spec.", s)
+
+        if ssc.update_modifier == UpdateModifier.FREEZE_INSTALLED:
+            for prec in ssc.prefix_data.iter_records():
+                if prec.name not in ssc.specs_map:
+                    if (prec.name not in conflict_specs and
+                            (prec.name not in explicit_pool or
+                             prec in explicit_pool[prec.name]) and
+                            # because it's not just immediate deps, but also
+                            # upstream things that matter, we must ensure
+                            # overlap for dependencies of things that have
+                            # otherwise passed our tests
+                            self._compare_pools(ssc, explicit_pool, prec.to_match_spec())):
+                        ssc.specs_map[prec.name] = prec.to_match_spec()
+                    else:
+                        ssc.specs_map[prec.name] = MatchSpec(
+                            prec.name, target=prec.to_match_spec(), optional=True)
+        log.debug("specs_map with targets: %s", ssc.specs_map)
 
         # If we're in UPDATE_ALL mode, we need to drop all the constraints attached to specs,
         # so they can all float and the solver can find the most up-to-date solution. In the case
@@ -411,15 +543,40 @@ class Solver(object):
         # they're no longer needed, and their presence would otherwise prevent the updated solution
         # the user most likely wants.
         if ssc.update_modifier == UpdateModifier.UPDATE_ALL:
-            ssc.specs_map = odict((pkg_name, MatchSpec(spec.name, optional=spec.optional))
-                                  for pkg_name, spec in iteritems(ssc.specs_map))
+            # history is preferable because it has explicitly installed stuff in it.
+            #   that simplifies our solution.
+            if ssc.specs_from_history_map:
+                ssc.specs_map = odict((spec, MatchSpec(spec))
+                                      if MatchSpec(spec).name not in
+                                      (_.name for _ in ssc.pinned_specs)
+                                      else (MatchSpec(spec).name,
+                                            ssc.specs_map[MatchSpec(spec).name])
+                                      for spec in ssc.specs_from_history_map
+                                      )
+                for prec in ssc.prefix_data.iter_records():
+                    # treat pip-installed stuff as explicitly installed, too.
+                    if prec.subdir == 'pypi':
+                        ssc.specs_map.update({prec.name: MatchSpec(prec.name)})
+            else:
+                ssc.specs_map = odict((prec.name, MatchSpec(prec.name))
+                                      if prec.name not in (_.name for _ in ssc.pinned_specs) else
+                                      (prec.name, ssc.specs_map[prec.name])
+                                      for prec in ssc.prefix_data.iter_records()
+                                      )
 
         # As a business rule, we never want to update python beyond the current minor version,
         # unless that's requested explicitly by the user (which we actively discourage).
-        if 'python' in ssc.specs_map:
+        if (any(_.name == 'python' for _ in ssc.solution_precs)
+                and not any(s.name == 'python' for s in self.specs_to_add)):
+
+            # will our prefix record conflict with any explict spec?  If so, don't add
+            #     anything here - let python float when it hasn't been explicitly specified
             python_prefix_rec = ssc.prefix_data.get('python')
-            if python_prefix_rec:
-                python_spec = ssc.specs_map['python']
+            if ('python' not in conflict_specs and
+                    ssc.update_modifier == UpdateModifier.FREEZE_INSTALLED):
+                ssc.specs_map['python'] = python_prefix_rec.to_match_spec()
+            else:
+                python_spec = ssc.specs_map.get('python', MatchSpec('python'))
                 if not python_spec.get('version'):
                     pinned_version = get_major_minor_version(python_prefix_rec.version) + '.*'
                     ssc.specs_map['python'] = MatchSpec(python_spec, version=pinned_version)
@@ -433,7 +590,7 @@ class Solver(object):
 
         # add in explicitly requested specs from specs_to_add
         # this overrides any name-matching spec already in the spec map
-        ssc.specs_map.update((s.name, s) for s in self.specs_to_add)
+        ssc.specs_map.update((s.name, s) for s in self.specs_to_add if s.name not in pin_overrides)
 
         # As a business rule, we never want to downgrade conda below the current version,
         # unless that's requested explicitly by the user (which we actively discourage).
@@ -441,14 +598,12 @@ class Solver(object):
             conda_prefix_rec = ssc.prefix_data.get('conda')
             if conda_prefix_rec:
                 conda_spec = ssc.specs_map['conda']
-                conda_in_specs_to_add_version = next(
-                    (spec.get('version') for spec in self.specs_to_add if spec.name == "conda"),
-                    None
-                )
+
+                conda_in_specs_to_add_version = ssc.specs_map.get('conda', {}).get('version')
                 if not conda_in_specs_to_add_version:
                     conda_spec = MatchSpec(conda_spec, version=">=%s" % conda_prefix_rec.version)
                 if context.auto_update_conda:
-                    conda_spec = MatchSpec(conda_spec, target=None)
+                    conda_spec = MatchSpec('conda', target=None)
                 ssc.specs_map['conda'] = conda_spec
 
         return ssc
@@ -458,7 +613,7 @@ class Solver(object):
         final_environment_specs = IndexedSet(concatv(
             itervalues(ssc.specs_map),
             ssc.track_features_specs,
-            ssc.pinned_specs,
+            # pinned specs removed here - added to specs_map in _add_specs instead
         ))
 
         # We've previously checked `solution` for consistency (which at that point was the
@@ -472,33 +627,50 @@ class Solver(object):
         # get_conflicting_specs() returns a "minimal unsatisfiable subset" which
         # may not be the only unsatisfiable subset. We may have to call get_conflicting_specs()
         # several times, each time making modifications to loosen constraints.
-        conflicting_specs = ssc.r.get_conflicting_specs(tuple(final_environment_specs))
+
+        conflicting_specs = set(ssc.r.get_conflicting_specs(final_environment_specs) or [])
+
         while conflicting_specs:
             specs_modified = False
             if log.isEnabledFor(DEBUG):
-                log.debug("conflicting specs: %s", dashlist(conflicting_specs))
+                log.debug("conflicting specs: %s", dashlist(
+                    s.target if s.target else s for s in conflicting_specs))
 
             # Are all conflicting specs in specs_map? If not, that means they're in
             # track_features_specs or pinned_specs, which we should raise an error on.
             specs_map_set = set(itervalues(ssc.specs_map))
             grouped_specs = groupby(lambda s: s in specs_map_set, conflicting_specs)
-            not_in_specs_map = grouped_specs.get(False, ())
-            if not_in_specs_map:
+            # force optional to true. This is what it is originally in
+            # pinned_specs, but we override that in _add_specs to make it
+            # non-optional when there's a name match in the explicit package
+            # pool
+            conflicting_pinned_specs = groupby(lambda s: MatchSpec(s, optional=True)
+                                               in ssc.pinned_specs, conflicting_specs)
+
+            if conflicting_pinned_specs.get(True):
                 in_specs_map = grouped_specs.get(True, ())
-                in_specs_map_and_specs_to_add = set(in_specs_map) & set(self.specs_to_add)
+                pinned_conflicts = conflicting_pinned_specs.get(True, ())
+                in_specs_map_or_specs_to_add = ((set(in_specs_map) | set(self.specs_to_add))
+                                                - set(pinned_conflicts))
+
                 raise SpecsConfigurationConflictError(
-                    sorted(s.__str__() for s in in_specs_map_and_specs_to_add),
-                    sorted(s.__str__() for s in not_in_specs_map),
+                    sorted(s.__str__() for s in in_specs_map_or_specs_to_add),
+                    sorted(s.__str__() for s in {s for s in pinned_conflicts}),
                     self.prefix
                 )
             for spec in conflicting_specs:
                 if spec.target and not spec.optional:
                     specs_modified = True
                     final_environment_specs.remove(spec)
-                    neutered_spec = MatchSpec(spec.name, target=spec.target, optional=True)
+                    if spec.get('version'):
+                        neutered_spec = MatchSpec(spec.name, version=spec.version)
+                    else:
+                        neutered_spec = MatchSpec(spec.name)
                     final_environment_specs.add(neutered_spec)
+                    ssc.specs_map[spec.name] = neutered_spec
             if specs_modified:
-                conflicting_specs = ssc.r.get_conflicting_specs(tuple(final_environment_specs))
+                conflicting_specs = set(ssc.r.get_conflicting_specs(
+                    tuple(final_environment_specs)))
             else:
                 # Let r.solve() use r.find_conflicts() to report conflict chains.
                 break
@@ -507,15 +679,22 @@ class Solver(object):
         if log.isEnabledFor(DEBUG):
             log.debug("final specs to add: %s",
                       dashlist(sorted(text_type(s) for s in final_environment_specs)))
-        ssc.solution_precs = ssc.r.solve(tuple(final_environment_specs))
+        ssc.solution_precs = ssc.r.solve(tuple(final_environment_specs),
+                                         specs_to_add=self.specs_to_add,
+                                         history_specs=ssc.specs_from_history_map)
 
         # add back inconsistent packages to solution
         if ssc.add_back_map:
             for name, (prec, spec) in iteritems(ssc.add_back_map):
-                if not any(d.name == name for d in ssc.solution_precs):
+                # spec here will only be set if the conflicting prec was in the original specs_map
+                #    if it isn't there, then we restore the conflict.  If it is there, though,
+                #    we keep the new, consistent solution
+                if not spec:
+                    # filter out solution precs and reinsert the conflict.  Any resolution
+                    #    of the conflict should be explicit (i.e. it must be in ssc.specs_map)
+                    ssc.solution_precs = [_ for _ in ssc.solution_precs if _.name != name]
                     ssc.solution_precs.append(prec)
-                    if spec:
-                        final_environment_specs.add(spec)
+                    final_environment_specs.add(spec)
 
         ssc.final_environment_specs = final_environment_specs
         return ssc
@@ -563,6 +742,14 @@ class Solver(object):
             # direct dependencies of flask.
             graph = PrefixGraph(ssc.solution_precs, self.specs_to_add)
             removed_nodes = graph.remove_youngest_descendant_nodes_with_specs()
+            self.specs_to_add = set(self.specs_to_add)
+            for prec in removed_nodes:
+                for dep in prec.depends:
+                    dep = MatchSpec(dep)
+                    if dep.name not in ssc.specs_map:
+                        self.specs_to_add.add(dep)
+            # unfreeze
+            self.specs_to_add = frozenset(self.specs_to_add)
 
             # Add back packages that are already in the prefix.
             specs_to_remove_names = set(spec.name for spec in self.specs_to_remove)
@@ -637,7 +824,8 @@ class Solver(object):
                     return
 
             conda_newer_precs = sorted(
-                SubdirData.query_all(conda_newer_spec, self.channels, self.subdirs),
+                SubdirData.query_all(conda_newer_spec, self.channels, self.subdirs,
+                                     repodata_fn=self._repodata_fn),
                 key=lambda x: VersionOrder(x.version)
                 # VersionOrder is fine here rather than r.version_key because all precs
                 # should come from the same channel
@@ -669,6 +857,7 @@ class Solver(object):
         if hasattr(self, '_index') and self._index:
             # added in install_actions for conda-build back-compat
             self._prepared_specs = prepared_specs
+            _supplement_index_with_system(self._index)
             self._r = Resolve(self._index, channels=self.channels)
         else:
             # add in required channels that aren't explicitly given in the channels list
@@ -684,8 +873,10 @@ class Solver(object):
                     additional_channels.add(Channel(channel))
 
             self.channels.update(additional_channels)
+
             reduced_index = get_reduced_index(self.prefix, self.channels,
-                                              self.subdirs, prepared_specs)
+                                              self.subdirs, prepared_specs, self._repodata_fn)
+            _supplement_index_with_system(reduced_index)
             self._prepared_specs = prepared_specs
             self._index = reduced_index
             self._r = Resolve(reduced_index, channels=self.channels)
@@ -696,6 +887,8 @@ class Solver(object):
     def _check_solution(self, ssc):
         # Ensure that solution is consistent with pinned specs.
         for spec in ssc.pinned_specs:
+            if spec.name in self.specs_to_add_names:
+                continue
             spec = MatchSpec(spec, optional=False)
             if not any(spec.match(d) for d in ssc.solution_precs):
                 # if the spec doesn't match outright, make sure there's no package by that
@@ -834,7 +1027,6 @@ def diff_for_unlink_link_precs(prefix, final_precs, specs_to_add=(), force_reins
 
 # def solve_prefix(prefix, r, specs_to_remove=(), specs_to_add=(), prune=False):
 #     # this function gives a "final state" for an existing prefix given just these simple inputs
-#     prune = context.prune or prune
 #     log.debug("solving prefix %s\n"
 #               "  specs_to_remove: %s\n"
 #               "  specs_to_add: %s\n"
