@@ -13,6 +13,7 @@ from os.path import join
 import sys
 from textwrap import dedent
 from traceback import format_exception, format_exception_only
+import getpass
 
 from . import CondaError, CondaExitZero, CondaMultiError, text_type
 from ._vendor.auxlib.entity import EntityEncoder
@@ -387,8 +388,13 @@ class CondaOSError(CondaError, OSError):
 
 class ProxyError(CondaError):
     def __init__(self, message):
-        msg = '%s' % message
-        super(ProxyError, self).__init__(msg)
+        message = dals("""
+        Conda cannot proceed due to an error in your proxy configuration.
+        Check for typos and other configuration errors in any '.netrc' file in your home directory,
+        any environment variables ending in '_PROXY', and any other system-wide proxy
+        configuration settings.
+        """)
+        super(ProxyError, self).__init__(message)
 
 
 class CondaIOError(CondaError, IOError):
@@ -488,20 +494,21 @@ class CouldntParseError(ParseError):
         super(CouldntParseError, self).__init__(self.args[0])
 
 
-class MD5MismatchError(CondaError):
-    def __init__(self, url, target_full_path, expected_md5sum, actual_md5sum):
+class ChecksumMismatchError(CondaError):
+    def __init__(self, url, target_full_path, checksum_type, expected_checksum, actual_checksum):
         message = dals("""
         Conda detected a mismatch between the expected content and downloaded content
         for url '%(url)s'.
           download saved to: %(target_full_path)s
-          expected md5 sum: %(expected_md5sum)s
-          actual md5 sum: %(actual_md5sum)s
+          expected %(checksum_type)s: %(expected_checksum)s
+          actual %(checksum_type)s: %(actual_checksum)s
         """)
         from .common.url import maybe_unquote
         url = maybe_unquote(url)
-        super(MD5MismatchError, self).__init__(message, url=url, target_full_path=target_full_path,
-                                               expected_md5sum=expected_md5sum,
-                                               actual_md5sum=actual_md5sum)
+        super(ChecksumMismatchError, self).__init__(
+            message, url=url, target_full_path=target_full_path, checksum_type=checksum_type,
+            expected_checksum=expected_checksum, actual_checksum=actual_checksum,
+        )
 
 
 class PackageNotInstalledError(CondaError):
@@ -608,50 +615,83 @@ class UnsatisfiableError(CondaError):
         Raises an exception with a formatted message detailing the
         unsatisfiable specifications.
     """
+    def _format_chain_str(self, bad_deps):
+        chains = {}
+        for dep in sorted(bad_deps, key=len, reverse=True):
+            dep1 = [s.partition(' ') for s in dep[1:]]
+            key = (dep[0],) + tuple(v[0] for v in dep1)
+            vals = ('',) + tuple(v[2] for v in dep1)
+            found = False
+            for key2, csets in iteritems(chains):
+                if key2[:len(key)] == key:
+                    for cset, val in zip(csets, vals):
+                        cset.add(val)
+                    found = True
+            if not found:
+                chains[key] = [{val} for val in vals]
+        for key, csets in iteritems(chains):
+            deps = []
+            for name, cset in zip(key, csets):
+                if '' not in cset:
+                    pass
+                elif len(cset) == 1:
+                    cset.clear()
+                else:
+                    cset.remove('')
+                    cset.add('*')
+                if name[0] == '@':
+                    name = 'feature:' + name[1:]
+                deps.append('%s %s' % (name, '|'.join(sorted(cset))) if cset else name)
+            chains[key] = ' -> '.join(deps)
+        bad_deps = [chains[key] for key in sorted(iterkeys(chains))]
+        return bad_deps
 
-    def __init__(self, bad_deps, chains=True):
+    def __init__(self, bad_deps, chains=True, strict=False):
         from .models.match_spec import MatchSpec
 
-        # Remove any target values from the MatchSpecs, convert to strings
-        bad_deps = [list(map(lambda x: str(MatchSpec(x, target=None)), dep)) for dep in bad_deps]
-        if chains:
-            chains = {}
-            for dep in sorted(bad_deps, key=len, reverse=True):
-                dep1 = [s.partition(' ') for s in dep[1:]]
-                key = (dep[0],) + tuple(v[0] for v in dep1)
-                vals = ('',) + tuple(v[2] for v in dep1)
-                found = False
-                for key2, csets in iteritems(chains):
-                    if key2[:len(key)] == key:
-                        for cset, val in zip(csets, vals):
-                            cset.add(val)
-                        found = True
-                if not found:
-                    chains[key] = [{val} for val in vals]
-            for key, csets in iteritems(chains):
-                deps = []
-                for name, cset in zip(key, csets):
-                    if '' not in cset:
-                        pass
-                    elif len(cset) == 1:
-                        cset.clear()
-                    else:
-                        cset.remove('')
-                        cset.add('*')
-                    if name[0] == '@':
-                        name = 'feature:' + name[1:]
-                    deps.append('%s %s' % (name, '|'.join(sorted(cset))) if cset else name)
-                chains[key] = ' -> '.join(deps)
-            bad_deps = [chains[key] for key in sorted(iterkeys(chains))]
-            msg = '''The following specifications were found to be in conflict:%s
-Use "conda search <package> --info" to see the dependencies for each package.'''
-        else:
-            bad_deps = [sorted(dep) for dep in bad_deps]
-            bad_deps = [', '.join(dep) for dep in sorted(bad_deps)]
-            msg = '''The following specifications were found to be incompatible with the
-others, or with the existing package set:%s
-Use "conda search <package> --info" to see the dependencies for each package.'''
-        msg = msg % dashlist(bad_deps)
+        messages = {'python': dals('''
+
+The following specifications were found
+to be incompatible with the existing python installation in your environment:\n{specs}
+
+If python is on the left-most side of the chain, that's the version you've asked for.
+When python appears to the right, that indicates that the thing on the left is somehow
+not available for the python version you've asked for.
+Your current python version is ({ref}).
+
+        '''),
+                    'request_conflict_with_history': dals('''
+
+The following specifications were found to be incompatible with a past
+explicit spec that is not an explicit spec in this operation ({ref}):\n{specs}
+
+                    '''),
+                    'direct': dals('''
+
+The following specifications were found to be incompatible with each other:\n{specs}
+
+''')}
+
+        msg = ""
+        for class_name, dep_class in bad_deps.items():
+            if dep_class:
+                _chains = []
+                for dep_chain, installed_blocker in dep_class:
+                    # Remove any target values from the MatchSpecs, convert to strings
+                    installed_blocker = str(MatchSpec(installed_blocker, target=None))
+                    dep_chain = [str(MatchSpec(dep, target=None)) for dep in dep_chain]
+                    _chains.append(dep_chain)
+
+                if _chains:
+                    _chains = self._format_chain_str(_chains)
+                else:
+                    _chains = [', '.join(c) for c in _chains]
+                msg += messages[class_name].format(specs=dashlist(_chains),
+                                                   ref=installed_blocker)
+        if strict:
+            msg += ('\nNote that strict channel priority may have removed '
+                    'packages required for satisfiability.')
+
         super(UnsatisfiableError, self).__init__(msg)
 
 
@@ -1227,7 +1267,11 @@ class ExceptionHandler(object):
             'User-Agent': self.user_agent,
         }
         _timeout = self.http_timeout
+        username = getpass.getuser()
+        error_report['is_ascii'] = True if all(ord(c) < 128 for c in username) else False
+        error_report['has_spaces'] = True if " " in str(username) else False
         data = json.dumps(error_report, sort_keys=True, cls=EntityEncoder) + '\n'
+        data = data.replace(str(username), "USERNAME_REMOVED")
         response = None
         try:
             # requests does not follow HTTP standards for redirects of non-GET methods
