@@ -4,32 +4,80 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import codecs
-from errno import EACCES, ELOOP, EPERM
+from errno import EACCES, EPERM, EROFS
 from io import open
 from logging import getLogger
 import os
 from os.path import basename, dirname, isdir, isfile, join, splitext
 from shutil import copyfileobj, copystat
 import sys
-import tarfile
 import tempfile
+import warnings as _warnings
+
+import conda_package_handling.api
 
 from . import mkdir_p
-from .delete import rm_rf, path_is_clean
+from .delete import path_is_clean, rm_rf
 from .link import islink, lexists, link, readlink, symlink
 from .permissions import make_executable
 from .update import touch
 from ... import CondaError
 from ..._vendor.auxlib.ish import dals
-from ...base.constants import PACKAGE_CACHE_MAGIC_FILE
+from ...base.constants import CONDA_PACKAGE_EXTENSION_V1, PACKAGE_CACHE_MAGIC_FILE
 from ...base.context import context
 from ...common.compat import on_win
 from ...common.path import ensure_pad, expand, win_path_double_escape, win_path_ok
 from ...common.serialize import json_dump
-from ...exceptions import (BasicClobberError, CaseInsensitiveFileSystemError, CondaOSError,
-                           maybe_raise, CondaFileIOError)
+from ...exceptions import BasicClobberError, CondaOSError, maybe_raise
 from ...models.enums import FileMode, LinkType
 
+
+# we have our own TemporaryDirectory implementation both for historical reasons and because
+#     using our rm_rf function is more robust than the shutil equivalent
+class TemporaryDirectory(object):
+    """Create and return a temporary directory.  This has the same
+    behavior as mkdtemp but can be used as a context manager.  For
+    example:
+
+        with TemporaryDirectory() as tmpdir:
+            ...
+
+    Upon exiting the context, the directory and everything contained
+    in it are removed.
+    """
+
+    # Handle mkdtemp raising an exception
+    name = None
+    _closed = False
+
+    def __init__(self, suffix="", prefix='tmp', dir=None):
+        self.name = tempfile.mkdtemp(suffix, prefix, dir)
+
+    def __repr__(self):
+        return "<{} {!r}>".format(self.__class__.__name__, self.name)
+
+    def __enter__(self):
+        return self.name
+
+    def cleanup(self, _warn=False, _warnings=_warnings):
+        from .delete import rm_rf as _rm_rf
+        if self.name and not self._closed:
+            try:
+                _rm_rf(self.name)
+            except (TypeError, AttributeError) as ex:
+                if "None" not in '%s' % (ex,):
+                    raise
+                _rm_rf(self.name)
+            self._closed = True
+
+    def __exit__(self, exc, value, tb):
+        self.cleanup()
+
+    def __del__(self):
+        # Issue a ResourceWarning if implicit cleanup needed
+        self.cleanup(_warn=True)
+
+
 log = getLogger(__name__)
 stdoutlog = getLogger('conda.stdoutlog')
 
@@ -154,7 +202,10 @@ class ProgressFileWrapper(object):
 
 def extract_tarball(tarball_full_path, destination_directory=None, progress_update_callback=None):
     if destination_directory is None:
-        destination_directory = tarball_full_path[:-8]
+        if tarball_full_path[-8:] == CONDA_PACKAGE_EXTENSION_V1:
+            destination_directory = tarball_full_path[:-8]
+        else:
+            destination_directory = tarball_full_path.splitext()[0]
     log.debug("extracting %s\n  to %s", tarball_full_path, destination_directory)
 
     # the most common reason this happens is due to hard-links, windows thinks
@@ -162,30 +213,10 @@ def extract_tarball(tarball_full_path, destination_directory=None, progress_upda
     #    have a .conda_trash extension though, so it's ok to just write into
     #    the same existing folder.
     if not path_is_clean(destination_directory):
-        log.debug("package folder {} was not empty, but we're writing there."
-                  .format(destination_directory))
+        log.debug("package folder %s was not empty, but we're writing there.",
+                  destination_directory)
 
-    with open(tarball_full_path, 'rb') as fileobj:
-        if progress_update_callback:
-            fileobj = ProgressFileWrapper(fileobj, progress_update_callback)
-        with tarfile.open(fileobj=fileobj) as tar_file:
-            if context.safety_checks:
-                for member in tar_file.getmembers():
-                    if (os.path.isabs(member.name) or
-                            not os.path.realpath(member.name).startswith(os.getcwd())):
-                        raise CondaFileIOError(tarball_full_path,
-                                               "contains unsafe path: {}".format(member.name))
-            try:
-                tar_file.extractall(path=destination_directory)
-            except EnvironmentError as e:
-                if e.errno == ELOOP:
-                    raise CaseInsensitiveFileSystemError(
-                        package_location=tarball_full_path,
-                        extract_location=destination_directory,
-                        caused_by=e,
-                    )
-                else:
-                    raise
+    conda_package_handling.api.extract(tarball_full_path, dest_dir=destination_directory)
 
     if sys.platform.startswith('linux') and os.getuid() == 0:
         # When extracting as root, tarfile will by restore ownership
@@ -408,7 +439,7 @@ def create_package_cache_directory(pkgs_dir):
         touch(join(pkgs_dir, PACKAGE_CACHE_MAGIC_FILE), mkdir=True, sudo_safe=sudo_safe)
         touch(join(pkgs_dir, 'urls'), sudo_safe=sudo_safe)
     except (IOError, OSError) as e:
-        if e.errno in (EACCES, EPERM):
+        if e.errno in (EACCES, EPERM, EROFS):
             log.trace("cannot create package cache directory '%s'", pkgs_dir)
             return False
         else:
@@ -428,7 +459,7 @@ def create_envs_directory(envs_dir):
         sudo_safe = expand(envs_dir).startswith(expand('~'))
         touch(join(envs_dir, envs_dir_magic_file), mkdir=True, sudo_safe=sudo_safe)
     except (IOError, OSError) as e:
-        if e.errno in (EACCES, EPERM):
+        if e.errno in (EACCES, EPERM, EROFS):
             log.trace("cannot create envs directory '%s'", envs_dir)
             return False
         else:
