@@ -214,7 +214,7 @@ else:  # pragma: no cover
         \\?\ in order to work with API calls.
         See http://msdn.microsoft.com/en-us/library/aa365247%28v=vs.85%29.aspx for
         details.
-        """
+        """  # NOQA
         if path.startswith('\\\\?\\'):
             return path
         path = abspath(path)
@@ -242,7 +242,7 @@ else:  # pragma: no cover
         path = _patch_path(path)
         try:
             return _is_symlink(next(find_files(path)))
-        except WindowsError as orig_error:
+        except WindowsError as orig_error:  # NOQA
             tmpl = "Error accessing {path}: {orig_error.message}"
             raise builtins.WindowsError(local_format(tmpl))
 
@@ -257,7 +257,7 @@ else:  # pragma: no cover
         >>> # This test might fail on a non-standard installation
         >>> 'Windows' in (fd.filename for fd in root_files)
         True
-        """
+        """  # NOQA
         fd = WIN32_FIND_DATA()
         handle = FindFirstFile(spec, byref(fd))
         while True:
